/******************************************************************************\
* ping.c - Simple ping utility using SOCK_RAW
* 
*       This is a part of the Microsoft Source Code Samples.
*       Copyright 1996-1997 Microsoft Corporation.
*       All rights reserved.
*       This source code is only intended as a supplement to
*       Microsoft Development Tools and/or WinHelp documentation.
*       See these sources for detailed information regarding the
*       Microsoft samples programs.
*
*		Notes:  Matt Kropp Feb. 2000
*			Fixed several bugs and memory leaks.
*			Modified to compile with LabWindows CVI
*
\******************************************************************************/

#pragma pack(4)

#define WIN32_LEAN_AND_MEAN

#include <winsock2.h>
#include "ping.h"

#include <stdio.h>
#include <stdlib.h>


SOCKET sockRaw;
char *icmp_data;
char *recvbuf;
struct sockaddr_in dest,from;


int ping(char *host, int datasize, int timeout)
{
	WSADATA wsaData;
	SOCKET sockRaw;
	struct sockaddr_in dest,from;
	struct hostent * hp;
	int bread, bwrote;
	int RoundTripTime;
	int fromlen = sizeof(from);
	char *dest_ip;
	char *icmp_data;
	char *recvbuf;
	unsigned int addr=0;
	USHORT seq_no = 0;
	
  
	if (WSAStartup(MAKEWORD(2,1),&wsaData) != 0)
	{
		printf("WSAStartup failed: %d\n",GetLastError());
		return -1;
	}

//
// WSA_FLAG_OVERLAPPED flag is required for SO_RCVTIMEO, SO_SNDTIMEO
// option. If NULL is used as last param for WSASocket, all I/O on the socket
// is synchronous, the internal user mode wait code never gets a chance to 
// execute, and therefore kernel-mode I/O blocks forever. A socket created 
// via the socket API has the overlapped I/O attribute set internally. But 
// here we need to use WSASocket to specify a RAW socket.
//
// If you want to use timeout with a synchronous non-overlapped socket created
// by WSASocket with last param set to NULL, you can set the timeout using the
// select API, or use WSAEventSelect and set the timeout in the 
// WSAWaitForMultipleEvents API.
//
	sockRaw = WSASocket(AF_INET,
						SOCK_RAW,
						IPPROTO_ICMP,
						NULL, 
						0,
						WSA_FLAG_OVERLAPPED);
  
	if (sockRaw == INVALID_SOCKET) 
	{
		printf("WSASocket() failed: %d\n",WSAGetLastError());
		WSACleanup();
		return -1;
	}

	// timout values less than 500 are interpreted as 500
	bread = setsockopt(sockRaw,SOL_SOCKET,SO_RCVTIMEO,(char*)&timeout,sizeof(timeout));
  
	if(bread == SOCKET_ERROR) 
	{
		printf("failed to set recv timeout: %d\n",WSAGetLastError());
		closesocket(sockRaw);
		WSACleanup();		
		return -1;
	}

	bread = setsockopt(sockRaw,SOL_SOCKET,SO_SNDTIMEO,(char*)&timeout,sizeof(timeout));
  
	if(bread == SOCKET_ERROR) 
	{
		printf("failed to set send timeout: %d\n",WSAGetLastError());
		closesocket(sockRaw);
		WSACleanup();		
		return -1;
	}
	memset(&dest,0,sizeof(dest));

	hp = gethostbyname(host);

	if (!hp)
	{
		addr = inet_addr(host);
	}
	if ((!hp)  && (addr == INADDR_NONE) ) 
	{
		printf("Unable to resolve %s\n",host);
		closesocket(sockRaw);
		WSACleanup();		
		return -1;
	}

	if (hp != NULL)
		memcpy(&(dest.sin_addr),hp->h_addr,hp->h_length);
	else
		dest.sin_addr.s_addr = addr;

	if (hp)
		dest.sin_family = hp->h_addrtype;
	else
		dest.sin_family = AF_INET;

	dest_ip = inet_ntoa(dest.sin_addr);

	if (datasize == 0)
		datasize = DEF_PACKET_SIZE;
	
	datasize += sizeof(IcmpHeader);  

	icmp_data = xmalloc(MAX_PACKET);
	if (!icmp_data)
	{
		printf("HeapAlloc failed %d\n",GetLastError());
		closesocket(sockRaw);
		WSACleanup();		
		return -1;
	}

	recvbuf = xmalloc(MAX_PACKET);

	if (!recvbuf)
	{
		printf("HeapAlloc failed %d\n",GetLastError());
		xfree(icmp_data);
		return -1;
	}
  

	memset(icmp_data,0,MAX_PACKET);
	fill_icmp_data(icmp_data,datasize);
	
	((IcmpHeader*)icmp_data)->i_cksum = 0;
	((IcmpHeader*)icmp_data)->timestamp = GetTickCount();

	((IcmpHeader*)icmp_data)->i_seq = seq_no++;
	((IcmpHeader*)icmp_data)->i_cksum = checksum((USHORT*)icmp_data, datasize);

	bwrote = sendto(sockRaw,icmp_data,datasize,0,(struct sockaddr*)&dest,sizeof(dest));
	
	if (bwrote == SOCKET_ERROR)
	{
		xfree(icmp_data);
		xfree(recvbuf);

		if (WSAGetLastError() == WSAETIMEDOUT) 
		{
			printf("timed out\n");
			closesocket(sockRaw);
			WSACleanup();
			return -1;
		}
		printf("sendto failed: %d\n",WSAGetLastError());
		closesocket(sockRaw);
		WSACleanup();		
		return -1;
	}
	if (bwrote < datasize ) 
	{
		printf("Wrote %d bytes\n",bwrote);
	}
	bread = recvfrom(sockRaw,recvbuf,MAX_PACKET,0,(struct sockaddr*)&from, &fromlen);
	if (bread == SOCKET_ERROR)
	{
		xfree(icmp_data);
		xfree(recvbuf);

		if (WSAGetLastError() == WSAETIMEDOUT) 
		{
			closesocket(sockRaw);
			WSACleanup();		
			printf("timed out\n");
			return -1;
		}
		printf("recvfrom failed: %d\n",WSAGetLastError());
		closesocket(sockRaw);
		WSACleanup();		
		return -1;
	}


	RoundTripTime = decode_resp(recvbuf,bread,&from);
	
	
	xfree(icmp_data);
	xfree(recvbuf);
	closesocket(sockRaw);
	WSACleanup();

	return RoundTripTime;
}
/* 
	The response is an IP packet. We must decode the IP header to locate 
	the ICMP data 
*/
int decode_resp(char *buf, int bytes,struct sockaddr_in *from) 
{
	IpHeader *iphdr;
	IcmpHeader *icmphdr;
	unsigned short iphdrlen;
	int time;

	iphdr = (IpHeader *)buf;

	iphdrlen = iphdr->h_len * 4 ; // number of 32-bit words *4 = bytes

	if (bytes  < iphdrlen + ICMP_MIN) 
	{
		printf("Too few bytes from %s\n",inet_ntoa(from->sin_addr));
	}

	icmphdr = (IcmpHeader*)(buf + iphdrlen);

	if (icmphdr->i_type != ICMP_ECHOREPLY) 
	{
		printf("non-echo type %d recvd\n",icmphdr->i_type);
		return -1;
	}
	if (icmphdr->i_id != (USHORT)GetCurrentProcessId()) 
	{
		printf("someone else's packet!\n");
		return -1;
	}
	time = (int) (GetTickCount() - icmphdr->timestamp);
	printf("%d bytes from %s:",bytes, inet_ntoa(from->sin_addr));
	printf(" icmp_seq = %d. ",icmphdr->i_seq);
	printf(" time: %d ms ",time);
	printf("\n");
	return time;	
}


USHORT checksum(USHORT *buffer, int size) 
{
	unsigned long cksum=0;

	while(size >1) 
	{
		cksum+=*buffer++;
		size -=sizeof(USHORT);
	}
  
	if(size ) 
	{
		cksum += *(UCHAR*)buffer;
	}

	cksum = (cksum >> 16) + (cksum & 0xffff);
	cksum += (cksum >>16);
	return (USHORT)(~cksum);
}
/* 
	Helper function to fill in various stuff in our ICMP request.
*/
void fill_icmp_data(char * icmp_data, int datasize)
{
	IcmpHeader *icmp_hdr;
	char *datapart;

	icmp_hdr = (IcmpHeader*)icmp_data;

	icmp_hdr->i_type = ICMP_ECHO;
	icmp_hdr->i_code = 0;
	icmp_hdr->i_id = (USHORT)GetCurrentProcessId();
	icmp_hdr->i_cksum = 0;
	icmp_hdr->i_seq = 0;
  
	datapart = icmp_data + sizeof(IcmpHeader);
	//
	// Place some junk in the buffer.
	//
	memset(datapart,'E', datasize - sizeof(IcmpHeader));
}
